-------------------------------------------------------------------------------
--   a_register.vhd
--
--   Copyright (C) 2008-2010 Atmark Techno, Inc.  All Rights Reserved.
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_misc.all;
library unisim;
use unisim.vcomponents.all;

entity a_register is
  generic (
    C_SLV_AWIDTH       : integer := 32;
    C_SLV_DWIDTH       : integer := 32;
    C_BASEADDR         : std_logic_vector := X"00000000";
    C_HIGHADDR         : std_logic_vector := X"00000000"
  );
  port (
    Bus2IP_Clk         : in  std_logic;
    Bus2IP_Reset       : in  std_logic;
    Bus2IP_Addr        : in  std_logic_vector(0 to C_SLV_AWIDTH-1);
    Bus2IP_RNW         : in  std_logic;
    IP2Bus_RdAck       : out std_logic;
    IP2Bus_WrAck       : out std_logic;
    IP2Bus_Ack         : out std_logic;
    IP2Bus_Error       : out std_logic;
    IP2Bus_Retry       : out std_logic;
    IP2Bus_ToutSup     : out std_logic;
    Bus2IP_Data        : in  std_logic_vector(0 to C_SLV_DWIDTH-1);
    Bus2IP_CS          : in  std_logic;
    Bus2IP_BE          : in  std_logic_vector(0 to C_SLV_DWIDTH/8-1);
    IP2Bus_Data        : out std_logic_vector(0 to C_SLV_DWIDTH-1);

    REC_RST            : out std_logic;                     -- REC_FIFO Reset
    PLAY_RST           : out std_logic;                     -- PLAY_FIFO Reset
    REC_FIFO_HFULL     : in  std_logic;                     -- REC_FIFO FULL Flag
    REC_FIFO_VALID     : in  std_logic;                     -- REC_FIFO Valid Flag
    PLAY_FIFO_FULL     : in  std_logic;                     -- PLAY_FIFO FULL Flag
    PLAY_FIFO_HEMP     : in  std_logic;                     -- PLAY_FIFO Half EMPTY Flag
    REC_FIFO_FULL      : in  std_logic;                     -- REC_FIFO FULL Flag
    PLAY_FIFO_EMP      : in  std_logic;                     -- PLAY_FIFO EMPTY Flag
    REC_FIFO_RD_EN     : out std_logic;                     -- REC_FIFO Read Enable
    PLAY_FIFO_WR_EN    : out std_logic;                     -- PLAY_FIFO Write Enable
    REC_FIFO_HFULL_INTR_EN    : out std_logic;              -- Interrupt Enable(REC_FIFO Half FULL)
    REC_FIFO_VALID_INTR_EN    : out std_logic;              -- Interrupt Enable(REC_FIFO Valid)
    PLAY_FIFO_FULL_INTR_EN    : out std_logic;              -- Interrupt Enable(PLAY_FIFO FULL)
    PLAY_FIFO_HEMP_INTR_EN    : out std_logic;              -- Interrupt Enable(PLAY_FIFO Half EMPTY)
    REC_DOUT           : in  std_logic_vector(31 downto 0); -- Data From REC_FIFO
    PLAY_FIFO_DIN      : out std_logic_vector(31 downto 0); -- Data To PLAY_FIFO
    MONO_SEL           : out std_logic;                     -- MONO/STEREO Select Signal 1:MONO 0:Stereo
    TYPE_SEL           : out std_logic_vector(0 to 1)
  );

end entity a_register;

architecture imp of a_register is

  signal comp             : std_logic;
  signal comp_sel         : std_logic;
  signal IP2Bus_Ack_w     : std_logic;
  signal IP2Bus_Ack_1wait : std_logic;
  signal IP2Bus_Ack_2wait : std_logic;

  signal REC_RST_w        : std_logic;
  signal PLAY_RST_w       : std_logic;

  signal REC_FIFO_HFULL_INTR_EN_w : std_logic;
  signal REC_FIFO_VALID_INTR_EN_w : std_logic;
  signal PLAY_FIFO_FULL_INTR_EN_w : std_logic;
  signal PLAY_FIFO_HEMP_INTR_EN_w : std_logic;

  signal rec_dout_w       : std_logic_vector(0 to 31);
  signal play_fifo_din_w  : std_logic_vector(0 to 31);

  signal MONO_SEL_w       : std_logic;
  signal TYPE_SEL_w       : std_logic_vector(0 to 1);
  signal byte_swap        : std_logic;


begin

IP2Bus_Retry   <= '0';
IP2Bus_Error   <= '0';
IP2Bus_ToutSup <= '0';

comp <= '1';
comp_sel <= comp and Bus2IP_CS;

process (Bus2IP_Clk) begin
  if Bus2IP_Clk'event and Bus2IP_Clk = '1' then
    if Bus2IP_Reset = '1' then
      IP2Bus_Ack_w <= '0';
      IP2Bus_Ack_1wait <= '0';
      IP2Bus_Ack_2wait <= '0';
    elsif (comp_sel = '1') and (IP2Bus_Ack_w = '0') then
      IP2Bus_Ack_1wait <= '1';
      IP2Bus_Ack_2wait <= IP2Bus_Ack_1wait;
      IP2Bus_Ack_w <= IP2Bus_Ack_2wait;
    else
      IP2Bus_Ack_1wait <= '0';
      IP2Bus_Ack_2wait <= '0';
      IP2Bus_Ack_w <= '0';
    end if;
  end if;
end process;

IP2Bus_Ack   <= IP2Bus_Ack_w;
IP2Bus_RdAck <= IP2Bus_Ack_w and Bus2IP_RNW;
IP2Bus_WrAck <= IP2Bus_Ack_w and not Bus2IP_RNW;

-----------------------------------------------------------------------------
-- Registers write
-----------------------------------------------------------------------------
-- x00 Control Register
--     bit30        REC_RST
--     bit31        PLAY_RST
-- x40 Interrupt Enable Register
--     bit28        REC_FIFO_HFULL_INTR_EN
--     bit29        REC_FIFO_VALID_INTR_EN
--     bit30        PLAY_FIFO_FULL_INTR_EN
--     bit31        PLAY_FIFO_HEMP_INTR_EN
-- x64 MONO/STEREO Select Register
--     bit31        MONO_SEL
-- x84 PLAY_FIFO
--     bit24 to 31  PLAY_FIFO_DIN
-----------------------------------------------------------------------------
  PLAY_FIFO_WR_EN <= '1' when (comp_sel  and not Bus2IP_RNW and IP2Bus_Ack_w) = '1'
                    and Bus2IP_Addr(24 to 31) = x"84" else '0';
  REC_FIFO_RD_EN  <= '1' when (comp_sel and Bus2IP_RNW and not IP2Bus_Ack_w 
                    and not IP2Bus_Ack_1wait and not IP2Bus_Ack_2wait) = '1' 
                    and Bus2IP_Addr(24 to 31) = x"80" else '0';

process (Bus2IP_Clk) begin
  if Bus2IP_Clk'event and Bus2IP_Clk = '1' then
      if Bus2IP_Reset = '1' then
        play_fifo_din_w <= x"00000000";
        MONO_SEL_w <= '0';
        TYPE_SEL_w <= "00";
        PLAY_RST_w <= '1';
        REC_RST_w <= '1';
        byte_swap <= '0';
      elsif (comp_sel = '1') and (Bus2IP_RNW = '0') and (IP2Bus_Ack_w = '1') then
        if Bus2IP_Addr(24 to 31) = x"00" then
          byte_swap <= Bus2IP_Data(29);
          REC_RST_w <= Bus2IP_Data(30);
          PLAY_RST_w <= Bus2IP_Data(31);
        end if;
        if Bus2IP_Addr(24 to 31) = x"40" then
          REC_FIFO_HFULL_INTR_EN_w <= Bus2IP_Data(28);
          REC_FIFO_VALID_INTR_EN_w <= Bus2IP_Data(29);
          PLAY_FIFO_FULL_INTR_EN_w <= Bus2IP_Data(30);
          PLAY_FIFO_HEMP_INTR_EN_w <= Bus2IP_Data(31);
        end if;
        if Bus2IP_Addr(24 to 31) = x"60" then
          TYPE_SEL_w(0) <= Bus2IP_Data(30);
          TYPE_SEL_w(1) <= Bus2IP_Data(31);
        end if;
        if Bus2IP_Addr(24 to 31) = x"64" then
          MONO_SEL_w <= Bus2IP_Data(31);
        end if;
        if Bus2IP_Addr(24 to 31) = x"84" then
          if byte_swap = '1' then
            play_fifo_din_w(24 to 31) <= Bus2IP_Data(16 to 23);
            play_fifo_din_w(16 to 23) <= Bus2IP_Data(24 to 31);
            play_fifo_din_w(8 to 15) <= Bus2IP_Data(0 to 7);
            play_fifo_din_w(0 to 7) <= Bus2IP_Data(8 to 15);
          else
            play_fifo_din_w <= Bus2IP_Data;
          end if;
        end if;
      end if;
  end if;
end process;

  PLAY_FIFO_DIN <= play_fifo_din_w;
  REC_RST <= REC_RST_w;
  PLAY_RST <= PLAY_RST_w;
  REC_FIFO_HFULL_INTR_EN <= REC_FIFO_HFULL_INTR_EN_w;
  REC_FIFO_VALID_INTR_EN <= REC_FIFO_VALID_INTR_EN_w;
  PLAY_FIFO_FULL_INTR_EN <= PLAY_FIFO_FULL_INTR_EN_w;
  PLAY_FIFO_HEMP_INTR_EN <= PLAY_FIFO_HEMP_INTR_EN_w;
  MONO_SEL <= MONO_SEL_w;
  TYPE_SEL <= TYPE_SEL_w;

-----------------------------------------------------------------------------
-- Registers read
-----------------------------------------------------------------------------
-- x00 Control Register
--     bit30        REC_RST
--     bit31        PLAY_RST
-- x10 Status Register
--     bit26        REC_FIFO_FULL
--     bit27        PLAY_FIFO_EMP
--     bit28        REC_FIFO_HFULL
--     bit29        REC_FIFO_VALID
--     bit30        PLAY_FIFO_FULL
--     bit31        PLAY_FIFO_HEMP
-- x40 Interrupt Enable Register
--     bit28        REC_FIFO_HFULL_INTR_EN
--     bit29        REC_FIFO_VALID_INTR_EN
--     bit30        PLAY_FIFO_FULL_INTR_EN
--     bit31        PLAY_FIFO_HEMP_INTR_EN
-- x64 MONO/STEREO Select Register
--     bit31        MONO_SEL
-- x80 REC_FIFO
--     bit24 to 31  REC_DOUT
-- xA0 PLAY_FIFO Depth
--     bit0  to 31  PLAY_FIFO_DEPTH
-- xA4 REC_FIFO Depth
--     bit0  to 31  REC_FIFO_DEPTH
-----------------------------------------------------------------------------
process (Bus2IP_Clk) begin
  if Bus2IP_Clk'event and Bus2IP_Clk = '1' then
    if Bus2IP_Reset = '1' then
      IP2Bus_Data <= x"00000000";
    elsif (comp_sel = '1') and (Bus2IP_RNW = '1') and (IP2Bus_Ack_w = '0') then
      if Bus2IP_Addr(24 to 31) = x"00" then
        IP2Bus_Data(0 to 28) <= (others => '0');
        IP2Bus_Data(29) <= byte_swap;
        IP2Bus_Data(30) <= REC_RST_w;
        IP2Bus_Data(31) <= PLAY_RST_w;
      end if;
      if Bus2IP_Addr(24 to 31) = x"10" then
        IP2Bus_Data(0 to 25) <= (others => '0');
        IP2Bus_Data(26) <= REC_FIFO_FULL;
        IP2Bus_Data(27) <= PLAY_FIFO_EMP;
        IP2Bus_Data(28) <= REC_FIFO_HFULL;
        IP2Bus_Data(29) <= REC_FIFO_VALID;
        IP2Bus_Data(30) <= PLAY_FIFO_FULL;
        IP2Bus_Data(31) <= PLAY_FIFO_HEMP;
      end if;
      if Bus2IP_Addr(24 to 31) = x"40" then
        IP2Bus_Data(0 to 27) <= (others => '0');
        IP2Bus_Data(28) <= REC_FIFO_HFULL_INTR_EN_w;
        IP2Bus_Data(29) <= REC_FIFO_VALID_INTR_EN_w;
        IP2Bus_Data(30) <= PLAY_FIFO_FULL_INTR_EN_w;
        IP2Bus_Data(31) <= PLAY_FIFO_HEMP_INTR_EN_w;
      end if;
      if Bus2IP_Addr(24 to 31) = x"60" then
        IP2Bus_Data(0 to 29) <= (others => '0');
        IP2Bus_Data(30) <= TYPE_SEL_w(0);
        IP2Bus_Data(31) <= TYPE_SEL_w(1);
      end if;
      if Bus2IP_Addr(24 to 31) = x"64" then
        IP2Bus_Data(0 to 30) <= (others => '0');
        IP2Bus_Data(31) <= MONO_SEL_w;
      end if;
      if Bus2IP_Addr(24 to 31) = x"80" then
          if byte_swap = '1' then
            IP2Bus_Data(24 to 31) <= rec_dout_w(16 to 23);
            IP2Bus_Data(16 to 23) <= rec_dout_w(24 to 31);
            IP2Bus_Data(8 to 15) <= rec_dout_w(0 to 7);
            IP2Bus_Data(0 to 7) <= rec_dout_w(8 to 15);
          else
            IP2Bus_Data(0 to 31) <= rec_dout_w;
          end if;
      end if;
      if Bus2IP_Addr(24 to 31) = x"A0" then
        IP2Bus_Data(0 to 31) <= x"00000400";
      end if;
      if Bus2IP_Addr(24 to 31) = x"A4" then
        IP2Bus_Data(0 to 31) <= x"00000400";
      end if;
      if Bus2IP_Addr(24 to 31) = x"FC" then
        IP2Bus_Data <= x"5008100D";
      end if;
    else
      IP2Bus_Data <= x"00000000";
    end if;
  end if;
end process;
  
  rec_dout_w <= REC_DOUT;

end architecture imp;
